@@ -16,6 +16,12 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	if(is.null(interceptLabel)) interceptLabel <- "(Intercept)"
 	nInts <- sum(attr(allTerms, "intercept"))
 
+	#XXX: use.ranef <- FALSE
+	#if(use.ranef && inherits(global.model, "mer")) {
+		#allTerms <- c(allTerms, paste("(", attr(allTerms0, "random.terms"), ")",
+			#sep = ""))
+	#}
+
 	if(length(grep(":", all.vars(reformulate(allTerms))) > 0L))
 		stop("variable names in the formula cannot contain \":\"")
 
@@ -31,7 +37,6 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 		}
 		#"For objects without a 'call' component the call to the fitting function \n",
 		#" must be used directly as an argument to 'dredge'.")
-
 		# NB: this is unlikely to happen:
 		if(!exists(as.character(gmCall[[1L]]), parent.frame(), mode="function"))
 			 stop("could not find function '", gmCall[[1L]], "'")
@@ -52,9 +57,6 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	}
 	logLik <- .getLogLik()
 
-	# TODO: other classes: model, fixed, etc...
-	gmCoefNames0 <- names(coeffs(global.model))
-
 	# Check for na.omit
 	if (!is.null(gmCall$na.action) &&
 		as.character(gmCall$na.action) %in% c("na.omit", "na.exclude")) {
@@ -64,7 +66,10 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	if(names(gmCall)[2L] == "") names(gmCall)[2L] <-
 		names(formals(deparse(gmCall[[1L]]))[1L])
 
-	gmCoefNames <- fixCoefNames(gmCoefNames0)
+	# TODO: other classes: model, fixed, etc...
+		#gmCoefNames0 <- names(coeffs(global.model))
+		#gmCoefNames <- fixCoefNames(gmCoefNames0)
+	gmCoefNames <- fixCoefNames(names(coeffs(global.model)))
 	#sglobCoefNames <- fixCoefNames(names(coeffs(global.model)))
 
 	n.vars <- length(allTerms)
@@ -73,7 +78,7 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 		warning("comparing models fitted by REML")
 
 	if (beta && is.null(tryCatch(beta.weights(global.model), error=function(e) NULL,
-		warning=function(e) NULL))) {
+		warning = function(e) NULL))) {
 		warning("do not know how to calculate beta weights for ",
 				class(global.model)[1L], ", argument 'beta' ignored")
 		beta <- FALSE
@@ -84,7 +89,7 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	# fixed variables:
 	if (!is.null(fixed)) {
 		if (inherits(fixed, "formula")) {
-			if (fixed[[1]] != "~" || length(fixed) != 2L)
+			if (fixed[[1L]] != "~" || length(fixed) != 2L)
 				warning("'fixed' should be a one-sided formula")
 			fixed <- c(getAllTerms(fixed))
 		} else if (!is.character(fixed)) {
@@ -99,26 +104,26 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	fixed <- c(fixed, allTerms[allTerms %in% interceptLabel])
 	n.fixed <- length(fixed)
 	termsOrder <- order(allTerms %in% fixed)
-	ordAllTerms <- allTerms[termsOrder]
-	allTerms <- ordAllTerms
-	isMER <- any(inherits(global.model, c("mer", "lmer", "glmer")))
-	gmFormula <- as.formula(formula(global.model))
-	gmFormulaEnv <- attr(gmFormula, ".Environment")
+	allTerms <- allTerms[termsOrder]
+	gmFormulaEnv <- environment(as.formula(formula(global.model), env = gmEnv))
+	# TODO: gmEnv <- gmFormulaEnv ???
 
 	### BEGIN:
 	## varying BEGIN
 	if(!missing(varying) && !is.null(varying)) {
-		variantsIdx <- expand.grid(lapply(varying, seq_along))
-		seq.variants <- seq.int(nrow(variantsIdx))
 		nvarying <- length(varying)
 		varying.names <- names(varying)
+		fvarying <- unlist(varying, recursive = FALSE)
+		vlen <- vapply(varying, length, 1L)
+		nvariants <- prod(vlen)
+		variants <- as.matrix(expand.grid(split(seq_len(sum(vlen)),
+			rep(seq_along(varying), vlen))))
 	} else {
-		variantsIdx <- NULL
-		seq.variants <- 1L
+		variants <- NULL
+		nvariants <- 1L
 		nvarying <- 0L
 		varying.names <- character(0L)
 	}
-	nvariants <- length(seq.variants)
 	## varying END
 
 	## extra BEGIN
@@ -153,15 +158,16 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	## extra END
 
 	nov <- as.integer(n.vars - n.fixed)
-	ncomb <- 2L ^ nov
+	ncomb <- (2L ^ nov) * nvariants
 
 	if(nov > 31L) stop(gettextf("maximum number of predictors is 31, but %d is given", nov))
-	if(nov > 10L) warning(gettextf("%d predictors will generate up to %.0f possible combinations", nov, ncomb))
+	#if(nov > 10L) warning(gettextf("%d predictors will generate up to %.0f combinations", nov, ncomb))
 	nmax <- ncomb * nvariants
 	if(evaluate) {
 		ret.nchunk <- 25L
 		ret.ncol <- n.vars + nvarying + 3L + nextra
 		ret <- matrix(NA_real_, ncol = ret.ncol, nrow = ret.nchunk)
+		retCoefTable <- vector(ret.nchunk, mode = "list")
 	} else {
 		ret.nchunk <- nmax
 	}
@@ -204,95 +210,99 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 
 	retColIdx <- if(nvarying) -n.vars - seq_len(nvarying) else TRUE
 
+	prevJComb <- 0L
 	for(iComb in seq.int(ncomb)) {
-		comb <- c(as.logical(intToBits(iComb - 1L)[comb.seq]), comb.sfx)
+		jComb <- ceiling(iComb / nvariants)
+		if(jComb != prevJComb) {
+			isok <- TRUE
+			prevJComb <- jComb
+			comb <- c(as.logical(intToBits(jComb - 1L)[comb.seq]), comb.sfx)
+			nvar <- sum(comb) - nInts
 
-		nvar <- sum(comb) - nInts
-		if(nvar > m.max || nvar < m.min) next;
-		if(hasSubset && !eval(subset, structure(as.list(comb), names=allTerms)))
-			next;
-
-		#terms1 <- allTerms[comb]
-		newArgs <- makeArgs(global.model, allTerms[comb], comb, argsOptions)
-
-		formulaList <- if(is.null(attr(newArgs, "formulaList"))) newArgs else
-			attr(newArgs, "formulaList")
-		if(!all(vapply(formulaList, formulaAllowed, logical(1L), marg.ex))) next;
-
-		if(!is.null(attr(newArgs, "problems"))) {
-			print.warnings(structure(vector(mode = "list",
-				length = length(attr(newArgs, "problems"))),
-					names = attr(newArgs, "problems")))
-		} # end if <problems>
-		# ivar j
-
-		cl <- gmCall
-		cl[names(newArgs)] <- newArgs
-
-		for (v in seq.variants) { ## --- Variants ---------------------------
-			modelId <- ((iComb - 1L) * nvariants) + v
-			clVariant <- cl
-			if(nvarying) {
-				newVaryingArgs <- sapply(varying.names, function(x)
-					varying[[x]][[variantsIdx[v, x]]], simplify = FALSE)
-				clVariant[varying.names] <- newVaryingArgs
+			if(nvar > m.max || nvar < m.min || (hasSubset &&
+				!eval(subset, structure(as.list(comb), names = allTerms)))) {
+				isok <- FALSE
+				next;
 			}
-
-			if(trace) {
-				cat(modelId, ": "); print(clVariant)
-				utils::flush.console()
+			newArgs <- makeArgs(global.model, allTerms[comb], comb, argsOptions)
+			formulaList <- if(is.null(attr(newArgs, "formulaList"))) newArgs else
+				attr(newArgs, "formulaList")
+			if(!all(vapply(formulaList, formulaAllowed, logical(1L), marg.ex)))  {
+				isok <- FALSE; next;
 			}
+			if(!is.null(attr(newArgs, "problems"))) {
+				print.warnings(structure(vector(mode = "list",
+					length = length(attr(newArgs, "problems"))),
+						names = attr(newArgs, "problems")))
+			} # end if <problems>
 
-			if(evaluate) {
-				# begin row1: (clVariant, gmEnv, modelId, IC(), applyExtras(),
-				#              nextra, allTerms, beta,
-				#              if(nvarying) variantsIdx[v] else NULL
-				fit1 <- tryCatch(eval(clVariant, gmEnv), error = function(err) {
-					err$message <- paste(conditionMessage(err), "(model",
-						modelId, "skipped)", collapse = "")
-					class(err) <- c("simpleError", "warning", "condition")
-					warning(err)
-					return(NULL)
-				})
-				if (is.null(fit1)) next;
+			cl <- gmCall
+			cl[names(newArgs)] <- newArgs
+		} #  end if(jComb != prevJComb)
 
-				if(nextra != 0L) {
-					extraResult1 <- applyExtras(fit1)
-					if(length(extraResult1) < nextra) {
-						tmp <- rep(NA_real_, nextra)
-						tmp[match(names(extraResult1), names(extraResult))] <- extraResult1
-						extraResult1 <- tmp
-					}
-					#row1 <- c(row1, extraResult1)
+		if(!isok) next;
+		## --- Variants ---------------------------
+		clVariant <- cl
+		if (nvarying) clVariant[varying.names] <- 
+			fvarying[variants[(iComb - 1L) %% nvariants + 1L, ]]
+
+		if(trace) {
+			cat(iComb, ": "); print(clVariant)
+			utils::flush.console()
+		}
+
+		if(evaluate) {
+			# begin row1: (clVariant, gmEnv, modelId, IC(), applyExtras(),
+			#              nextra, allTerms, beta,
+			#              if(nvarying) variantsIdx[v] else NULL
+			fit1 <- tryCatch(eval(clVariant, gmEnv), error = function(err) {
+				err$message <- paste(conditionMessage(err), "(model",
+					iComb, "skipped)", collapse = "")
+				class(err) <- c("simpleError", "warning", "condition")
+				warning(err)
+				return(NULL)
+			})
+
+			if (is.null(fit1)) next;
+
+			if(nextra != 0L) {
+				extraResult1 <- applyExtras(fit1)
+				if(length(extraResult1) < nextra) {
+					tmp <- rep(NA_real_, nextra)
+					tmp[match(names(extraResult1), names(extraResult))] <- extraResult1
+					extraResult1 <- tmp
 				}
-				ll <- logLik(fit1)
-				row1 <- c(
-					matchCoef(fit1, all.terms = allTerms, beta = beta)[allTerms],
-					extraResult1, df = attr(ll, "df"), ll = ll, ic = IC(fit1)
-				)
-				## end -> row1
+				#row1 <- c(row1, extraResult1)
+			}
 
-				k <- k + 1L # all OK, add model to table
+			mcoef1 <- matchCoef(fit1, all.terms = allTerms, beta = beta,
+				allCoef = TRUE)
 
+			ll <- logLik(fit1)
+			row1 <- c(mcoef1[allTerms], extraResult1,
+				df = attr(ll, "df"), ll = ll, ic = IC(fit1)
+			)
+			## end -> row1
 
-				ret.nrow <- nrow(ret)
-				if(k > ret.nrow) {
-					nadd <- min(ret.nchunk, nmax - ret.nrow)
+			k <- k + 1L # all OK, add model to table
 
+			ret.nrow <- nrow(ret)
+			if(k > ret.nrow) { # append if necesarry
+				nadd <- min(ret.nchunk, nmax - ret.nrow)
+				retCoefTable <- c(retCoefTable, vector(nadd, mode = "list"))
 				ret <- rbind(ret, matrix(NA, ncol = ret.ncol, nrow = nadd),
 					deparse.level = 0L)
 					calls <- c(calls, vector("list", nadd))
-					ord <- c(ord, integer(nadd))
-				}
-				ord[k] <- modelId
-
-				ret[k, retColIdx] <- row1
-			} else { # if evaluate
-				k <- k + 1L # all OK, add model to table
+				ord <- c(ord, integer(nadd))
 			}
-			calls[[k]] <- clVariant
 
-		} # for (v ...)
+			ord[k] <- iComb
+			ret[k, retColIdx] <- row1
+			retCoefTable[[k]] <- attr(mcoef1, "coefTable")
+		} else { # if evaluate
+			k <- k + 1L # all OK, add model to table
+		}
+		calls[[k]] <- clVariant
 	} ### for (iComb ...)
 
 	if(k == 0L) stop("the result is empty")
@@ -304,11 +314,12 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 		ret <- ret[i, , drop = FALSE]
 		ord <- ord[i]
 		calls <- calls[i]
+		retCoefTable <- retCoefTable[i]
 	}
 
 	if(nvarying) {
 		varlev <- ord %% nvariants; varlev[varlev == 0L] <- nvariants
-		ret[, n.vars + seq_len(nvarying) ] <- as.matrix(variantsIdx)[varlev, ]
+		ret[, n.vars + seq_len(nvarying)] <- variants[varlev, ]
 	}
 
 	ret <- as.data.frame(ret)
@@ -317,44 +328,42 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	# Convert columns with presence/absence of terms to factors
 	tfac <- which(!(allTerms %in% gmCoefNames))
 
-	ret[tfac] <- lapply(ret[tfac], factor, levels=NaN, labels="+")
+	ret[tfac] <- lapply(ret[tfac], factor, levels = NaN, labels="+")
 
 	i <- seq_along(allTerms)
 	v <- order(termsOrder)
 	ret[, i] <- ret[, v]
-	#ret[, seq_along(allTerms)] <- ret[, order(termsOrder)]
 	allTerms <- allTerms[v]
-	#colnames(ret) <- c(allTerms0, varying.names, "df", "logLik", ICName)
 	colnames(ret) <- c(allTerms, varying.names, extraNames, "df", "logLik", ICName)
 
 	if(nvarying) {
 		variant.names <- lapply(varying, function(x)
 			make.unique(if(is.null(names(x))) as.character(x) else names(x)))
 		for (i in varying.names) ret[, i] <-
-			factor(ret[, i], levels = seq_along(variant.names[[i]]),
-				labels = variant.names[[i]])
+			factor(ret[, i], labels = variant.names[[i]])
 	}
 
 	o <- order(ret[, ICName], decreasing = FALSE)
 	ret <- ret[o, ]
+	retCoefTable <- retCoefTable[o]
+
 	ret$delta <- ret[, ICName] - min(ret[, ICName])
 	ret$weight <- exp(-ret$delta / 2) / sum(exp(-ret$delta / 2))
 
-	ret <- structure(ret,
+	structure(ret,
 		class = c("model.selection", "data.frame"),
 		calls = calls[o],
 		global = global.model,
 		global.call = gmCall,
-		terms = allTerms,
+		terms = structure(allTerms, interceptLabel = interceptLabel),
 		rank = IC,
-		rank.call = attr(IC,"call"),
-		call = match.call(expand.dots = TRUE)
+		rank.call = attr(IC, "call"),
+		beta = beta,
+		call = match.call(expand.dots = TRUE),
+		coefTables = retCoefTable,
+		nobs = nobs(global.model),
+		vCols = varying.names
 	)
-
-	if (!is.null(attr(allTerms0, "random.terms")))
-		attr(ret, "random.terms") <- attr(allTerms0, "random.terms")
-
-	return(ret)
 } ######
 
 `subset.model.selection` <-
@@ -379,7 +388,7 @@ function(x, subset, select, recalc.weights = TRUE, ...) {
 function (x, i, j, recalc.weights = TRUE, ...) {
 	ret <- `[.data.frame`(x, i, j, ...)
 	if (missing(j)) {
-		s <- c("row.names", "calls")
+		s <- c("row.names", "calls", "coefTables", "random.terms", "order")
 		k <- match(dimnames(ret)[[1L]], dimnames(x)[[1L]])
 		attrib <- attributes(x)
 		attrib[s] <- lapply(attrib[s], `[`, k)
@@ -399,48 +408,97 @@ function (x, i, j, recalc.weights = TRUE, ...) {
 
 `print.model.selection` <-
 function(x, abbrev.names = TRUE, warnings = getOption("warn") != -1L, ...) {
-	if(!is.null(x$weight))
-		x$weight <- round(x$weight, 3L)
+	orig.x <- x
+	if(!is.null(x$weight)) x$weight <- round(x$weight, 3L)
 	xterms <- attr(x, "terms")
 	if(is.null(xterms)) {
 		print.data.frame(x, ...)
 	} else {
-		xterms <- gsub(" ", "", xterms)
+		xterms <- gsub(" ", "", xterms, fixed = TRUE)
 		if(abbrev.names) xterms <- abbreviateTerms(xterms, 3L)
 
 		colnames(x)[seq_along(xterms)] <-  xterms
-		cl <- attr(x, "global.call")
-		if(!is.null(cl)) {
+
+		globcl <- attr(x, "global.call")
+		if(!is.null(globcl)) {
 			cat("Global model call: ")
-			print(cl)
+			print(globcl)
 			cat("---\n")
-		}
+			random.terms <- attr(getAllTerms(attr(x, "global")), "random.terms")
+			if(!is.null(random.terms)) random.terms <- list(random.terms)
+		} else random.terms <- attr(x, "random.terms")
 
 		cat("Model selection table \n")
-		dig <- c("R^2" = 4L, df = 0L, logLik = 3L, AICc = 1L, AICc = 1L, AIC = 1L,
-			BIC = 1L, QAIC = 1L, QAICc = 1L, ICOMP = 1L, Cp = 1L, delta = 2L, weight = 3L)
+		dig <- c("R^2" = 4L, df = 0L, logLik = 3L, AICc = 1L, AICc = 1L,
+			AIC = 1L, BIC = 1L, QAIC = 1L, QAICc = 1L, ICOMP = 1L, Cp = 1L,
+			delta = 2L,	weight = 3L)
 
 		j <- match(colnames(x), names(dig), nomatch = 0L)
 		i <- sapply(x, is.numeric) & (j == 0L)
 		x[, i] <- signif(x[, i], 4L)
 		for(i in names(dig)[j]) x[, i] <- round(x[, i], digits = dig[i])
 
+		vLegend <- character(0L)
+		if(abbrev.names) {
+			vCols <- attr(x, "vCols")
+			vlen <- nchar(vCols)
+			i <- vCols[1L]
+
+			if(!is.null(vCols)) {
+				for(i in vCols) {
+					z <- x[, i]
+					lev <- levels(z)
+					lev <- lev[!(lev %in% c("", "NULL"))]
+					z <- factor(z, levels = lev)
+					shlev <- gsub("((?<=F)ALSE|(?<=T)RUE|~(1\\|)?| +)", "", lev,
+						perl = TRUE, ignore.case = TRUE)
+					shlev <- abbreviate(shlev, nchar(i))
+					#shlev <- paste(substr(i, 1, 1), seq(nlevels(z)), sep="")
+					x[, i] <- factor(z, labels = shlev)
+					if(any(j <- shlev != lev)) vLegend <- c(vLegend, paste(i,
+						": ", paste(shlev[j], "=", sQuote(lev[j]),
+						collapse = ", "), sep = ""))
+				}
+			}
+		}
+
+		uqran <- unique(unlist(random.terms, use.names = FALSE))
+		abbran <- abbreviate(gsub("1 | ", "", uqran, fixed = TRUE), 1L)
+		colran <- vapply(random.terms, function(s) paste(abbran[match(s, uqran)],
+			collapse = "+"), "")
+
+		if(addrandcol <- length(unique(colran)) > 1L) {
+			k <- which(colnames(x) == "df")[1L]
+			x <- cbind(x[, 1L:(k - 1L)], random = colran, x[, k:ncol(x)])
+		}
+
 		print.default(as.matrix(x)[, !sapply(x, function(.x) all(is.na(.x))),
 			drop = FALSE], na.print = "", quote = FALSE)
-		if (!is.null(attr(x, "random.terms"))) {
-			cat("Random terms:", paste(attr(x, "random.terms"), collapse=", "),
-				"\n")
+
+		if(abbrev.names && length(vLegend))
+			cat("Abbreviations:", vLegend, sep = "\n")
+
+		if(!is.null(random.terms)) {
+			if(addrandcol) {
+				cat("Random terms: \n")
+				cat(paste(abbran, "=", sQuote(uqran)), sep = "\n")
+			} else {
+				cat("Random terms (all models): \n")
+				cat(paste(sQuote(uqran)), sep = ", ")
+				cat("\n")
+			}
+
 		}
 		if (warnings && !is.null(attr(x, "warnings"))) {
 			cat("\n"); print.warnings(attr(x, "warnings"))
 		}
 	}
+	invisible(orig.x)
 }
 
 `update.model.selection` <- function (object, global.model, ..., evaluate = TRUE) {
     cl <- attr(object, "call")
-    if (is.null(cl))
-        stop("need an object with call component")
+    if (is.null(cl)) stop("need an object with call component")
     extras <- match.call(expand.dots = FALSE)$...
 
 	if(!missing(global.model))
@@ -459,3 +517,13 @@ function(x, abbrev.names = TRUE, warnings = getOption("warn") != -1L, ...) {
 
 `coef.model.selection` <- function (object, ...)
 	object[, attr(object, "terms")]
+
+`logLik.model.selection` <- function (object, ...) {
+	nobs <- attr(object, "nobs")
+	n <- nrow(object)
+	ret <- vector(n, mode = "list")
+	for(i in 1:n) ret[[i]] <-
+		structure(object[i, "logLik"], df = object[i, "df"], nobs = nobs,
+			class = "logLik")
+	ret
+}
