@@ -1,17 +1,110 @@
 `model.avg` <-
+function (object, ..., revised.var = TRUE) {
+	if (isTRUE("method" %in% names(match.call())))
+		stop("argument 'method' is no longer accepted")
+	UseMethod("model.avg")
+}
+
+
+`model.avg.model.selection` <-
+function(object, subset, fit = FALSE, ..., revised.var = TRUE) {
+
+	if(!missing(subset)) {
+		cl <- match.call()
+		cl[[1L]] <- as.name("subset")
+		names(cl)[2L] <- "x"
+		object <- eval(cl[1L:3L], parent.frame())
+	}
+	if(fit || length(list(...))) {
+		cl <- match.call()
+		cl$fit <- NULL
+		arg1 <- names(cl)[-(1L:2L)] %in% names(formals("model.avg.default"))
+		cl1 <- cl[c(TRUE, TRUE, !arg1)]
+		cl1[[1L]] <- as.name("get.models")
+		cl2 <- cl[c(TRUE, TRUE, arg1)]
+		cl2[[2L]] <- cl1
+		cl2[[1L]] <- as.name("model.avg")
+		#message("recreating the model objects")
+		return(eval(cl2, parent.frame()))
+	}
+
+	ct <- attr(object, "coefTables")
+	#coefNames <- if(!is.null(attr(object, "global")))
+		#names(coeffs(attr(object, "global"))) else
+	coefNames <- fixCoefNames(unique(unlist(lapply(ct, rownames),
+		use.names = FALSE)), sort = TRUE)
+
+	nCoef <- length(coefNames)
+	nModels <- length(ct)
+	coefArray <- array(NA_real_, dim = c(nModels, 3L, nCoef),
+		dimnames = list(NULL, c("Estimate", "Std. Error", "df"), coefNames))
+	for(i in seq_along(ct)) {
+		z <- ct[[i]]
+		coefArray[i, 1:3, ]
+		coefArray[i, seq_len(ncol(z)), ] <- t(z[match(coefNames, rownames(z)), ])
+	}
+	weight <- object$weight / sum(object$weight)
+
+	avgcoef <- array(dim = c(nCoef, 5L), dimnames = list(coefNames, c("Estimate",
+		"Std. Error", "Adjusted SE", "Lower CI", "Upper CI")))
+	for(i in seq_len(nCoef))
+		avgcoef[i, ] <- par.avg(coefArray[, 1L, i], coefArray[, 2L, i], weight,
+			df = coefArray[, 3L, i], revised.var = revised.var)
+
+
+	missing.par <- is.na(coefArray[, 1L, ])
+	coef.shrink <- avgcoef[, 1L] *
+		colSums(array(weight * as.double(!missing.par), dim = c(nModels, nCoef)))
+
+	#allterms1 <- lapply(attr(object, "calls"), function(x)
+		#getAllTerms(as.formula(x[[switch(as.character(x[[1L]]),
+			#lme=, lme.formula= "fixed", gls= "model", "formula")]])))
+
+	all.terms <- attr(object, "terms")
+	all.vterms <- all.terms[!(all.terms %in% attr(all.terms, "interceptLabel")
+		| apply(is.na(object[, all.terms]), 2L, all))]
+
+	allterms1 <- apply(!is.na(object[, all.vterms, drop = FALSE]), 1L, function(x) all.vterms[x])
+
+
+	#all.terms <- unique(unlist(allterms1))
+	all.model.names <- .modelNames(allTerms = allterms1, uqTerms = all.vterms)
+
+	mstab <- object[, -(seq_len(ncol(object) - 5L))]
+	colnames(mstab)[4:5] <- c("Delta", "Weight")
+	rownames(mstab) <- all.model.names
+
+	avgcoef[is.nan(avgcoef)] <- NA
+
+	ret <- list(
+		summary = as.data.frame(mstab),
+		term.codes = attr(all.model.names, "variables"),
+		avg.model = avgcoef,
+		coef.shrinkage = coef.shrink,
+		coefArray = coefArray,
+		importance = importance(object),
+		beta = attr(object, "beta"),
+		term.names = coefNames,
+		x = NULL,
+		residuals = NULL,
+		formula = if(!is.null(attr(object, "global")))
+			formula(attr(object, "global")) else NULL,
+		call = match.call()
+	)
+
+	attr(ret, "beta") <- attr(object, "beta")
+	attr(ret, "nobs") <- attr(object, "nobs")
+	attr(ret, "revised.var") <- revised.var
+	class(ret) <- "averaging"
+	return(ret)
+}
+
+
+`model.avg.default` <-
 function(object, ..., beta = FALSE,
 	rank = NULL, rank.args = NULL, revised.var = TRUE,
 	dispersion = NULL) {
 
-	if (isTRUE("method" %in% names(match.call())))
-		stop("the argument 'method' is no longer accepted")
-
-	if(inherits(object, "model.selection")) {
-		if(!("subset" %in% names(match.call(get.models))))
-			warning("'subset' argument is missing. Using all models by default")
-		object <- get.models(object, ...)
-	}
-
 	if (inherits(object, "list")) {
 		models <- object
 		object <- object[[1L]]
@@ -23,8 +116,8 @@ function(object, ..., beta = FALSE,
         rank <- .getRank(rank, rank.args = rank.args, object = object)
 	}
 
-	nmodels <- length(models)
-	if(nmodels == 1L) stop("only one model supplied. Nothing to do")
+	nModels <- length(models)
+	if(nModels == 1L) stop("only one model supplied. Nothing to do")
 	.checkModels(models)
 
     alpha <- 0.05
@@ -32,7 +125,8 @@ function(object, ..., beta = FALSE,
 	ICname <- deparse(attr(rank,"call")[[1L]])
 
 	allterms1 <- lapply(models, getAllTerms)
-	all.terms <- unique(unlist(allterms1))
+	all.terms <- unique(unlist(allterms1, use.names = FALSE))
+
 	# sort by level (main effects first)
 	all.terms <- all.terms[order(vapply(gregexpr(":", all.terms),
 		function(x) if(x[1L] == -1L) 0L else length(x), numeric(1L)), all.terms)]
@@ -62,53 +156,51 @@ function(object, ..., beta = FALSE,
 	ic <- vapply(models, rank, numeric(1L))
 	#dev <- if (!is.null(tryCatch(deviance(models[[1L]]), error = .fnull)))
 		#vapply(models, deviance, numeric(1L)) else NA
-	ll <- vapply(models, logLik, numeric(1L))
+	logLiks <- lapply(models, logLik)
+	#ll <- vapply(models, logLik, numeric(1L))
 	delta <- ic - min(ic)
 	weight <- exp(-delta / 2) / sum(exp(-delta / 2))
 	model.order <- order(weight, decreasing = TRUE)
 
-	#if(!is.null(dispersion)) dispersion <- rep(dispersion, length.out = nmodels)
+	#if(!is.null(dispersion)) dispersion <- rep(dispersion, length.out = nModels)
 
 	# DEBUG:
 	# sapply(sapply(sapply(models[model.order], coef), names), paste, collapse="+")
 	# sapply(models, function(x) paste(match(getAllTerms(x), all.terms), collapse="+"))
 
 	# ----!!! From now on, everything MUST BE ORDERED by 'weight' !!!-----------
-	mstab <- cbind(logLik = ll, IC = ic, Delta = delta, Weight = weight,
-		deparse.level = 0)
+	mstab <- cbind(df = vapply(logLiks, attr, numeric(1L), "df"),
+		logLik = as.numeric(logLiks), IC = ic, Delta = delta, Weight = weight,
+		deparse.level = 0L)
 	rownames(mstab) <- all.model.names
 	mstab <- mstab[model.order, ]
 	if(!is.null(dispersion)) {
-		dispersion <- rep(dispersion, length.out = nmodels)[model.order]
+		dispersion <- rep(dispersion, length.out = nModels)[model.order]
 		mstab <- cbind(mstab, Dispersion = dispersion)
 	}
 
 	weight <- mstab[, "Weight"] # has been sorted in table
 	models <- models[model.order]
 
-	allCoefNames <- unique(unlist(lapply(mcoeffs, names)))
-	allCoefNames <- allCoefNames[order(vapply(gregexpr(":", allCoefNames),
-		function(x) if(x[1L] == -1L) 0L else length(x), numeric(1L)), allCoefNames)]
-	npar <- length(allCoefNames)
+	allCoefNames <- fixCoefNames(unique(unlist(lapply(mcoeffs, names))), sort = TRUE)
+	#allCoefNames <- allCoefNames[order(!(allCoefNames %in% interceptLabel))]
+	nCoef <- length(allCoefNames)
 
 	if (beta)	response.sd <- sd(model.response(model.frame(object)))
 
-	all.coef <- all.se <- all.df <- matrix(NA_real_, nrow = nmodels, ncol = npar)
-	for (i in seq_len(nmodels)) {
+	coefArray <- array(NA_real_, dim = c(nModels, 3L, nCoef),
+		dimnames = list(rownames(mstab), c("Estimate", "Std. Error", "df"), allCoefNames))
+
+	for (i in seq_len(nModels)) {
 		m <- models[[i]]
 		coefmat <- coefTable(m, dispersion = dispersion[i])
-		ncoef <- NROW(coefmat)
-		if(ncoef > 0L) {
-			#if(is.null(mdf) || !length(mdf)) mdf <- rep(NA, ncoef)
+		rownames(coefmat) <- fixCoefNames(rownames(coefmat))
+		if(NROW(coefmat) > 0L) {
 			if (beta) coefmat[, 1L:2L] <- coefmat[, 1L:2L] * sd(model.matrix(m)) / response.sd
 			j <- match(rownames(coefmat), allCoefNames)
-			all.coef[i, j] <- coefmat[, 1L]
-			all.se[i, j] <- coefmat[, 2L]
-			mdf <- coefDf(m)[!is.na(coeffs(m))]
-			if(!is.null(mdf) && length(mdf)) all.df[i, j] <- mdf
+			coefArray[i, 1L:3L, j] <- t(coefmat)
 		}
 	}
-	rownames(all.se) <- rownames(all.coef) <- rownames(mstab)
 
 	# Benchmark: 3.7x faster
 	#system.time(for(i in 1:10000) t(array(unlist(p), dim=c(length(all.terms),length(models)))))
@@ -121,32 +213,28 @@ function(object, ..., beta = FALSE,
 	names(importance) <- all.terms
 	importance <- sort(importance, decreasing = TRUE)
 
-	missing.par <- is.na(all.coef)
-
-	avg.model <- t(vapply(seq_along(allCoefNames),
+	avgcoef <- t(vapply(seq_along(allCoefNames),
 		function(i) par.avg(
-			all.coef[, i],
-			se = all.se[, i],
+			coefArray[, 1L, i],
+			se = coefArray[, 2L, i],
 			weight = weight,
-			df = all.df[, i],
+			df = coefArray[, 3L, i],
 			alpha = alpha,
-			revised.var = revised.var),
-		double(5L)))
-
-	avg.model[is.nan(avg.model)] <- NA
+			revised.var = revised.var), double(5L)))
 
 
-	dimnames(avg.model) <- list(allCoefNames, c("Estimate", "Std. Err.",
+	avgcoef[is.nan(avgcoef)] <- NA
+
+	dimnames(avgcoef) <- list(allCoefNames, c("Estimate", "Std. Error",
 		"Adjusted SE", "Lower CI", "Upper CI"))
 
-	colnames(all.coef) <- colnames(all.se) <- colnames(all.df) <- allCoefNames
     names(all.terms) <- seq_along(all.terms)
+	colnames(mstab)[3L] <- ICname
 
-	colnames(mstab)[2L] <- ICname
-
-	coef.shrink <- avg.model[, 1] *
+	missing.par <- is.na(coefArray[, 1L, ])
+	coef.shrink <- avgcoef[, 1L] *
 		#colSums(weight * !is.na(all.coef))
-		colSums(array(weight * as.double(!missing.par), dim = dim(all.coef)))
+		colSums(array(weight * as.double(!missing.par), dim = c(nModels, nCoef)))
 
 	#global.mm <- model.matrix(fit)
 	#cnmmxx2 <- unique(unlist(lapply(gm, function(x) names(coef(x)))))
@@ -169,29 +257,24 @@ function(object, ..., beta = FALSE,
 	frm <- reformulate(all.terms,
 				response = attr(trm, "variables")[-1L][[attr(trm, "response")]])
 
-	#print(colSums(weight) * array(as.double(!is.na(all.coef)), dim=dim(all.coef)))
-
 	ret <- list(
 		summary = as.data.frame(mstab),
-		coefficients = all.coef,
-		se = all.se,
-		#variable.codes2 = all.terms,
 		term.codes = attr(all.model.names, "variables"),
-		avg.model = avg.model,
+		avg.model = avgcoef,
 		coef.shrinkage = coef.shrink,
+		coefArray = coefArray,
 		importance = importance,
 		beta = beta,
 		term.names = allCoefNames,
 		x = mmxs,
 		residuals = rsd,
 		formula = frm,
-		call = match.call(),
-		dfs = all.df
+		call = match.call()
 	)
 
 	attr(ret, "modelList") <- models
-	#attr(ret, "method") <- method # c("Coefficient" = method, "Variance" = method.var)
 	attr(ret, "beta") <- beta
+	attr(ret, "nobs") <- nobs(object)
 	attr(ret, "revised.var") <- revised.var
 	class(ret) <- "averaging"
 	return(ret)
@@ -201,7 +284,6 @@ function(object, ..., beta = FALSE,
 function(object, full = FALSE, ...) if(full) object$coef.shrinkage else
 	object$avg.model[, 1L]
 
-
 # TODO: predict type="response" + average on response scale
 `predict.averaging` <-
 function(object, newdata = NULL, se.fit = FALSE, interval = NULL,
@@ -211,6 +293,7 @@ function(object, newdata = NULL, se.fit = FALSE, interval = NULL,
 	if (!missing(interval)) .NotYetUsed("interval", error = FALSE)
 
 	models <- attr(object, "modelList")
+	if(is.null(models)) stop("cannot predict without model list")
 
 	# Benchmark: vapply is ~4x faster
 	#system.time(for(i in 1:1000) sapply(models, inherits, what="gam")) /
@@ -262,7 +345,7 @@ function(object, newdata = NULL, se.fit = FALSE, interval = NULL,
 		cl <- as.call(cl)
 		pred <- lapply(models, function(x) {
 			cl[[2L]] <- x
-			tryCatch(eval(cl), error=function(e) e)
+			tryCatch(eval(cl), error = function(e) e)
 		})
 
 		err <- sapply(pred, inherits, "condition")
@@ -328,29 +411,28 @@ function (object, ...) {
 	no.ase <- all(is.na(cf[, 3L]))
 	z <- abs(cf[,1] / cf[, if(no.ase) 2L else 3L])
 	pval <- 2 * pnorm(z, lower.tail = FALSE)
-	coefmat <- cbind(cf[, if(no.ase) 1L:2L else 1L:3L],
+	object$coefmat <- cbind(cf[, if(no.ase) 1L:2L else 1L:3L],
 					 `z value` = z,
 					 `Pr(>|z|)` = zapsmall(pval))
-	object$coefmat <- coefmat
-	structure(object, class=c("summary.averaging", "averaging"))
+	structure(object, class = c("summary.averaging", "averaging"))
 }
 
 `confint.averaging` <-
 function (object, parm, level = 0.95, ...) {
-    cf <- object$coefficients
+    cf <- object$coefArray[, 1L, ]
     pnames <- colnames(cf)
     if (missing(parm)) parm <- pnames
 		else if (is.numeric(parm)) parm <- pnames[parm]
 
     a2 <- 1 - level
     a <- a2 / 2
-    se <- object$se
+    se <- object$coefArray[, 2L, ]
     wts <- object$summary$Weight
-    dfs <- object$dfs
+    dfs <- object$coefArray[, 3L, ]
     ci <- t(sapply(parm, function(i)
-		par.avg(cf[,i], se[,i], wts, object$dfs[, i], alpha = a2)))[, 4L:5L]
+		par.avg(cf[,i], se[,i], wts, dfs[, i], alpha = a2)))[, 4L:5L]
     pct <- stats:::format.perc(c(a, 1L - a), 3L)
-	
+
 	ci[is.na(object$coef.shrinkage), ] <- NA_real_
     colnames(ci) <- pct
     return(ci)
@@ -386,7 +468,7 @@ function (x, digits = max(3L, getOption("digits") - 3L),
 	#if (no.ase) cat("Confidence intervals are unadjusted \n")
 
 	cat("\nFull model-averaged coefficients (with shrinkage):", "\n")
-	printCoefmat(matrix(x$coef.shrink, nrow = 1L,
+	printCoefmat(matrix(x$coef.shrinkage, nrow = 1L,
 		dimnames = list("", x$term.names)), P.values = FALSE,
 		has.Pvalue = FALSE, cs.ind = seq_along(x$term.names), tst.ind = NULL)
 
@@ -410,6 +492,8 @@ function(x, ...) {
 `vcov.averaging` <- function (object, ...) {
 	full <- FALSE
 	models <- attr(object, "modelList")
+	if(is.null(models)) stop("cannot predict without model list")
+
 	vcovs <- lapply(lapply(models, vcov), as.matrix)
 	names.all <- object$term.names
 	nvars <- length(names.all)
@@ -419,14 +503,16 @@ function(x, ...) {
 
 	#vcov0 <- matrix(NA, nrow=nvars, ncol = nvars,	dimnames = list(names.all, names.all))
 	vcov0 <- matrix(if(full) 0 else NA, nrow = nvars,
-		ncol=nvars,	dimnames = list(names.all, names.all))
+		ncol = nvars, dimnames = list(names.all, names.all))
+
+	v <- vcovs[[1]]
 
 	vcovs2 <- lapply(vcovs, function(v) {
 		i <- match(dimnames(v)[[1L]], names.all)
-		vcov0[i,i] <- v
+		vcov0[i, i] <- v
 		return(vcov0)
 	})
-	b1 <- object$coefficients
+	b1 <- object$coefArray[, 1L, ]
 	if(full) b1[is.na(b1)] <- 0
 
 	avgb <- object$avg.model[, 1L]
@@ -440,6 +526,16 @@ function(x, ...) {
 	return(res)
 }
 
-`logLik.averaging` <- function (object, ...)
-	return(structure(lapply(attr(object, "modelList"), .getLogLik()),
-			  names = rownames(object$summary)))
+`logLik.averaging` <- function (object, ...) {
+	models <- attr(object, "modelList")
+	if(is.null(models)) {
+		nobs <- attr(object, "nobs")
+		apply(object$summary, 1L, function(x) structure(list(x[2L]),
+			df = x[1L], nobs = nobs, class = "logLik"))
+	} else {
+		structure(lapply(attr(object, "modelList"), .getLogLik()),
+			names = rownames(object$summary))
+	}
+}
+
+`coefTable.averaging` <- function (model, ...)  model$avg.model[, 1:2]
